@@ -1,10 +1,10 @@
 # -*- encoding: utf-8 -*-
 #
-# Copyright (c) 2013 Paul Tagliamonte <paultag@debian.org>
+# Copyright (c) 2013, 2014 Paul Tagliamonte <paultag@debian.org>
 # Copyright (c) 2013 Julien Danjou <julien@danjou.info>
 # Copyright (c) 2013 Nicolas Dandrimont <nicolas.dandrimont@crans.org>
 # Copyright (c) 2013 James King <james@agentultra.com>
-# Copyright (c) 2013 Bob Tolbert <bob@tolbert.org>
+# Copyright (c) 2013, 2014 Bob Tolbert <bob@tolbert.org>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -24,7 +24,6 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 # DEALINGS IN THE SOFTWARE.
 
-from hy.models.lambdalist import HyLambdaListKeyword
 from hy.models.expression import HyExpression
 from hy.models.keyword import HyKeyword
 from hy.models.integer import HyInteger
@@ -48,6 +47,7 @@ import importlib
 import codecs
 import ast
 import sys
+import keyword
 
 from collections import defaultdict
 
@@ -74,6 +74,21 @@ def load_stdlib():
             _stdlib[e] = module
 
 
+# True, False and None included here since they
+# are assignable in Python 2.* but become
+# keywords in Python 3.*
+def _is_hy_builtin(name, module_name):
+    extras = ['True', 'False', 'None',
+              'true', 'false', 'nil', 'null']
+    if name in extras or keyword.iskeyword(name):
+        return True
+    # for non-Hy modules, check for pre-existing name in
+    # _compile_table
+    if not module_name.startswith("hy."):
+        return name in _compile_table
+    return False
+
+
 _compile_table = {}
 
 
@@ -305,13 +320,12 @@ def _raise_wrong_args_number(expression, error):
                                len(expression)))
 
 
-def checkargs(exact=None, min=None, max=None, even=None):
+def checkargs(exact=None, min=None, max=None, even=None, multiple=None):
     def _dec(fn):
         def checker(self, expression):
             if exact is not None and (len(expression) - 1) != exact:
                 _raise_wrong_args_number(
                     expression, "`%%s' needs %d arguments, got %%d" % exact)
-
             if min is not None and (len(expression) - 1) < min:
                 _raise_wrong_args_number(
                     expression,
@@ -330,6 +344,14 @@ def checkargs(exact=None, min=None, max=None, even=None):
                     "`%%s' needs an %s number of arguments, got %%d"
                     % (even_str))
 
+            if multiple is not None:
+                if not (len(expression) - 1) in multiple:
+                    choices = ", ".join([str(val) for val in multiple[:-1]])
+                    choices += " or %s" % multiple[-1]
+                    _raise_wrong_args_number(
+                        expression,
+                        "`%%s' needs %s arguments, got %%d" % choices)
+
             return fn(self, expression)
 
         return checker
@@ -430,6 +452,7 @@ class HyASTCompiler(object):
 
     def _parse_lambda_list(self, exprs):
         """ Return FunctionDef parameter values from lambda list."""
+        ll_keywords = ("&rest", "&optional", "&key", "&kwargs")
         ret = Result()
         args = []
         defaults = []
@@ -439,10 +462,7 @@ class HyASTCompiler(object):
 
         for expr in exprs:
 
-            if isinstance(expr, HyLambdaListKeyword):
-                if expr not in expr._valid_types:
-                    raise HyTypeError(expr, "{0} is not a valid "
-                                      "lambda-keyword.".format(repr(expr)))
+            if expr in ll_keywords:
                 if expr == "&rest" and lambda_keyword is None:
                     lambda_keyword = expr
                 elif expr == "&optional":
@@ -616,7 +636,7 @@ class HyASTCompiler(object):
 
             return imports, ret.replace(form), False
 
-        elif isinstance(form, (HySymbol, HyLambdaListKeyword)):
+        elif isinstance(form, HySymbol):
             return imports, HyExpression([HySymbol(name),
                                           HyString(form)]).replace(form), False
 
@@ -665,13 +685,22 @@ class HyASTCompiler(object):
 
     @builds("throw")
     @builds("raise")
-    @checkargs(max=1)
+    @checkargs(multiple=[0, 1, 3])
     def compile_throw_expression(self, expr):
         expr.pop(0)
         ret = Result()
         if expr:
             ret += self.compile(expr.pop(0))
 
+        cause = None
+        if len(expr) == 2 and expr[0] == HyKeyword(":from"):
+            if not PY3:
+                raise HyCompileError(
+                    "raise from only supported in python 3")
+            expr.pop(0)
+            cause = self.compile(expr.pop(0))
+            cause = cause.expr
+
         # Use ret.expr to get a literal `None`
         ret += ast.Raise(
             lineno=expr.start_line,
@@ -680,7 +709,7 @@ class HyASTCompiler(object):
             exc=ret.expr,
             inst=None,
             tback=None,
-            cause=None)
+            cause=cause)
 
         return ret
 
@@ -707,7 +736,8 @@ class HyASTCompiler(object):
                              lineno=expr.start_line,
                              col_offset=expr.start_column)
 
-        returnable = Result(expr=expr_name, temp_variables=[expr_name, name])
+        returnable = Result(expr=expr_name, temp_variables=[expr_name, name],
+                            contains_yield=body.contains_yield)
 
         body += ast.Assign(targets=[name],
                            value=body.force_expr,
@@ -732,25 +762,10 @@ class HyASTCompiler(object):
                 handler_results += self._compile_catch_expression(e, name)
                 handlers.append(handler_results.stmts.pop())
             elif e[0] == HySymbol("else"):
-                if orelse:
-                    raise HyTypeError(
-                        e,
-                        "`try' cannot have more than one `else'")
-                else:
-                    orelse = self._compile_branch(e[1:])
-                    # XXX tempvar magic
-                    orelse += orelse.expr_as_stmt()
-                    orelse = orelse.stmts
+                orelse = self.try_except_helper(e, HySymbol("else"), orelse)
             elif e[0] == HySymbol("finally"):
-                if finalbody:
-                    raise HyTypeError(
-                        e,
-                        "`try' cannot have more than one `finally'")
-                else:
-                    finalbody = self._compile_branch(e[1:])
-                    # XXX tempvar magic
-                    finalbody += finalbody.expr_as_stmt()
-                    finalbody = finalbody.stmts
+                finalbody = self.try_except_helper(e, HySymbol("finally"),
+                                                   finalbody)
             else:
                 raise HyTypeError(e, "Unknown expression in `try'")
 
@@ -768,8 +783,8 @@ class HyASTCompiler(object):
                 col_offset=expr.start_column,
                 type=None,
                 name=None,
-                body=[ast.Pass(lineno=expr.start_line,
-                               col_offset=expr.start_column)])]
+                body=[ast.Raise(lineno=expr.start_line,
+                                col_offset=expr.start_column)])]
 
         ret = handler_results
 
@@ -809,6 +824,17 @@ class HyASTCompiler(object):
             body=body,
             orelse=orelse) + returnable
 
+    def try_except_helper(self, hy_obj, symbol, accumulated):
+        if accumulated:
+            raise HyTypeError(
+                hy_obj,
+                "`try' cannot have more than one `%s'" % symbol)
+        else:
+            accumulated = self._compile_branch(hy_obj[1:])
+            accumulated += accumulated.expr_as_stmt()
+            accumulated = accumulated.stmts
+        return accumulated
+
     @builds("except")
     @builds("catch")
     def magic_internal_form(self, expr):
@@ -1003,7 +1029,10 @@ class HyASTCompiler(object):
     @checkargs(max=1)
     def compile_yield_expression(self, expr):
         expr.pop(0)
-        ret = Result(contains_yield=True)
+        if PY33:
+            ret = Result(contains_yield=False)
+        else:
+            ret = Result(contains_yield=True)
 
         value = None
         if expr != []:
@@ -1516,7 +1545,7 @@ class HyASTCompiler(object):
     def compile_require(self, expression):
         """
         TODO: keep track of what we've imported in this run and then
-        "unimport" it after we've completed `thing' so that we don't polute
+        "unimport" it after we've completed `thing' so that we don't pollute
         other envs.
         """
         expression.pop(0)
@@ -1754,19 +1783,26 @@ class HyASTCompiler(object):
 
     def _compile_assign(self, name, result,
                         start_line, start_column):
+
+        str_name = "%s" % name
+        if _is_hy_builtin(str_name, self.module_name):
+            raise HyTypeError(name,
+                              "Can't assign to a builtin: `%s'" % str_name)
+
         result = self.compile(result)
-
-        if result.temp_variables and isinstance(name, HyString):
-            result.rename(name)
-            return result
-
         ld_name = self.compile(name)
-        st_name = self._storeize(ld_name)
 
-        result += ast.Assign(
-            lineno=start_line,
-            col_offset=start_column,
-            targets=[st_name], value=result.force_expr)
+        if result.temp_variables \
+           and isinstance(name, HyString) \
+           and '.' not in name:
+            result.rename(name)
+        else:
+            st_name = self._storeize(ld_name)
+            result += ast.Assign(
+                lineno=start_line,
+                col_offset=start_column,
+                targets=[st_name],
+                value=result.force_expr)
 
         result += ld_name
         return result
@@ -1859,7 +1895,7 @@ class HyASTCompiler(object):
         ret, args, defaults, stararg, kwargs = self._parse_lambda_list(arglist)
 
         if PY34:
-            # Python 3.4+ requres that args are an ast.arg object, rather
+            # Python 3.4+ requires that args are an ast.arg object, rather
             # than an ast.Name or bare string.
             args = [ast.arg(arg=ast_str(x),
                             annotation=None,  # Fix me!
@@ -1967,12 +2003,12 @@ class HyASTCompiler(object):
             except TypeError:
                 raise HyTypeError(
                     expression,
-                    "Wrong argument type for defclass slots definition.")
+                    "Wrong argument type for defclass attributes definition.")
             for b in body_expression:
                 if len(b) != 2:
                     raise HyTypeError(
                         expression,
-                        "Wrong number of argument in defclass slot.")
+                        "Wrong number of argument in defclass attribute.")
                 body += self._compile_assign(b[0], b[1],
                                              b.start_line, b.start_column)
                 body += body.expr_as_stmt()
